@@ -30,6 +30,7 @@
 #include <errno.h>
 #include <signal.h>
 #include <ctype.h>
+#include <time.h>
 
 #if defined(WINDOWS)
 #include <tchar.h>
@@ -37,7 +38,8 @@
 #include <io.h>
 #include <conio.h>
 #include <math.h>
-#elif defined(AIX)
+#else
+#if defined(AIX)
 #define _BOOL
 #include <unistd.h>
 #include <curses.h>
@@ -45,6 +47,7 @@
 #include <unistd.h>
 #include <curses.h>
 #endif
+#endif
 
 #include "porting.h"
 
@@ -73,6 +76,46 @@ aix_malloc (size_t size)
 #endif
 
 #if defined (WINDOWS)
+/*
+ * realpath() -
+ *    return: pointer to resolved_path or NULL if error occurred
+ *    path(in): the relative path to be resolved
+ *    resolved_path(out): the buffer to output resolved path
+ */
+char *
+realpath (const char *path, char *resolved_path)
+{
+  struct stat stat_buf;
+  char *tmp_str = _fullpath (resolved_path, path, _MAX_PATH);
+  char tmp_path[_MAX_PATH] = { 0 };
+  int len = 0;
+
+  if (tmp_str != NULL)
+    {
+      strncpy (tmp_path, tmp_str, _MAX_PATH);
+
+      /* 
+       * The output of _fullpath() ends with '\'(Windows format) or without it. 
+       * It doesn't end with '/'(Linux format).
+       * 
+       * Even if the directory path exists, the stat() in Windows fails when
+       * the directory path ends with '\'.
+       */
+      len = strlen (tmp_path);
+      if (len > 0 && tmp_path[len - 1] == '\\')
+        {
+          tmp_path[len - 1] = '\0';
+        }
+
+      if (stat (tmp_path, &stat_buf) == 0)
+        {
+          return tmp_str;
+        }
+    }
+
+  return NULL;
+}
+
 /*
  * poll() -
  *    return: return poll result
@@ -87,7 +130,8 @@ poll (struct pollfd *fds, nfds_t nfds, int timeout)
   fd_set rset, wset, eset;
   fd_set *rp, *wp, *ep;
   unsigned long int i;
-  int r, max_fd;
+  int r;
+  unsigned int max_fd;
 
   tp = NULL;
   if (timeout >= 0)
@@ -1009,6 +1053,23 @@ cub_dirname_r (const char *path, char *pathbuf, size_t buflen)
   return (int) len;
 }
 
+#if defined(AIX)
+#undef ceil
+double
+aix_ceil (double x)
+{
+  double result = ceil (x);
+  /* e.g ceil(-0.5) should be -0, in AIX, it is 0 */
+  if ((x < 0) && (result == 0))
+    {
+      result = -result;
+    }
+  return result;
+}
+
+#define ceil(x) aix_ceil(x)
+#endif
+
 #if !defined(HAVE_DIRNAME)
 char *
 dirname (const char *path)
@@ -1092,6 +1153,39 @@ basename (const char *path)
 }
 #endif /* !HAVE_BASENAME */
 
+#if defined(WINDOWS)
+char *
+ctime_r (const time_t * time, char *time_buf)
+{
+  int err;
+  assert (time != NULL && time_buf != NULL);
+
+  err = ctime_s (time_buf, CTIME_MAX, time);
+  if (err != 0)
+    {
+      return NULL;
+    }
+  return time_buf;
+}
+#endif /* !WINDOWS */
+
+#if defined(WINDOWS)
+struct tm *
+localtime_r (const time_t * time, struct tm *tm_val)
+{
+  int err;
+  assert (time != NULL && tm_val != NULL);
+
+  err = localtime_s (tm_val, time);
+  if (err != 0)
+    {
+      return NULL;
+    }
+  return tm_val;
+}
+#endif /* WIDNOWS */
+
+
 #if defined (ENABLE_UNUSED_FUNCTION)
 int
 utona (unsigned int u, char *s, size_t n)
@@ -1909,11 +2003,216 @@ win32_exchange64 (UINT64 volatile *ptr, UINT64 new_val)
   return old;
 }
 #endif /* _WIN64 */
+
 #endif /* WINDOWS */
 
+#if defined(WINDOWS)
+/*
+ * strtod_win () convert string to double
+ * return : the converted double
+ * str (in): string to convert
+ * end_ptr (in): see strtod
+ */
+double
+strtod_win (const char *str, char **end_ptr)
+{
+  bool is_hex = false;
+  double result = 0.0, int_d = 0.0, float_d = 0.0;
+  double tmp_d = 0.0;
+  const char *p = NULL, *dot_p = NULL, *end_p = NULL;
+  int sign_flag = 1;
+
+  if (str == NULL || *str == '\0')
+    {
+      if (end_ptr != NULL)
+	{
+	  *end_ptr = (char *) str;
+	}
+      return result;
+    }
+
+  /* if the string start with "0x", "0X", "+0x", "+0X", "-0x" or "-0X" 
+   * then deal with it as hex string
+   */
+  p = str;
+  if (*p == '+')
+    {
+      p++;
+    }
+  else if (*p == '-')
+    {
+      sign_flag = -1;
+      p++;
+    }
+
+  if (*p == '0' && (*(p + 1) == 'x' || *(p + 1) == 'X'))
+    {
+      is_hex = true;
+      p += 2;
+    }
+
+  if (is_hex)
+    {
+      /* convert integer part */
+      while (*p != '\0')
+	{
+	  if (*p == '.')
+	    {
+	      break;
+	    }
+
+	  if ('0' <= *p && *p <= '9')
+	    {
+	      tmp_d = (double) (*p - '0');
+	    }
+	  else if ('A' <= *p && *p <= 'F')
+	    {
+	      tmp_d = (double) (*p - 'A' + 10);
+	    }
+	  else if ('a' <= *p && *p <= 'f')
+	    {
+	      tmp_d = (double) (*p - 'a' + 10);
+	    }
+	  else
+	    {
+	      end_p = p;
+	      goto end;
+	    }
+
+	  int_d = int_d * 16.0 + tmp_d;
+
+	  p++;
+	}
+      end_p = p;
+
+      /* convert float part */
+      if (*p == '.')
+	{
+	  /* find the end */
+	  dot_p = p;
+	  while (*++p != '\0')
+	    ;
+	  end_p = p;
+	  p--;
+
+	  while (p != dot_p)
+	    {
+	      if ('0' <= *p && *p <= '9')
+		{
+		  tmp_d = (double) (*p - '0');
+		}
+	      else if ('A' <= *p && *p <= 'F')
+		{
+		  tmp_d = (double) (*p - 'A' + 10);
+		}
+	      else if ('a' <= *p && *p <= 'f')
+		{
+		  tmp_d = (double) (*p - 'a' + 10);
+		}
+	      else
+		{
+		  end_p = p;
+		  goto end;
+		}
+
+	      float_d = (float_d + tmp_d) / 16.0;
+
+	      p--;
+	    }
+	}
+
+      result = int_d + float_d;
+      if (sign_flag == -1)
+	{
+	  result = -result;
+	}
+
+      /* underflow and overflow */
+      if (result > DBL_MAX || (-result) > DBL_MAX)
+	{
+	  errno = ERANGE;
+	}
+    }
+  else
+    {
+      result = strtod (str, end_ptr);
+    }
+
+end:
+
+  if (is_hex && end_ptr != NULL)
+    {
+      *end_ptr = (char *) end_p;
+    }
+
+  return result;
+}
+#endif
+
+/*
+ * timeval_diff_in_msec -
+ *
+ *   return: msec
+ *
+ */
+INT64
+timeval_diff_in_msec (const struct timeval * end_time,
+		      const struct timeval * start_time)
+{
+  INT64 msec;
+
+  msec = (end_time->tv_sec - start_time->tv_sec) * 1000LL;
+  msec += (end_time->tv_usec - start_time->tv_usec) / 1000LL;
+
+  return msec;
+}
+
+/*
+ * timeval_add_msec -
+ *   return: 0
+ *
+ *   addted_time(out):
+ *   start_time(in):
+ *   msec(in):
+ */
+int
+timeval_add_msec (struct timeval *added_time,
+		  const struct timeval *start_time, int msec)
+{
+  added_time->tv_sec = start_time->tv_sec + msec / 1000LL;
+
+  if (start_time->tv_usec + ((msec % 1000LL) * 1000LL) >= 1000000LL)
+    {
+      added_time->tv_sec += 1;
+    }
+  added_time->tv_usec = (start_time->tv_usec +
+			 ((msec % 1000LL) * 1000LL) % 10000000LL);
+
+  return 0;
+}
+
+/*
+ * timeval_to_timespec -
+ *   return: 0
+ *
+ *   to(out):
+ *   from(in):
+ */
+int
+timeval_to_timespec (struct timespec *to, const struct timeval *from)
+{
+  assert (to != NULL);
+  assert (from != NULL);
+
+  to->tv_sec = from->tv_sec;
+  to->tv_nsec = from->tv_usec * 1000LL;
+
+  return 0;
+}
+
 
 /*
- * port_open_memstream - make memory stream file handle if possible.
+ * port_open_memstream - make memory stream file handle if possible. 
  *			 if not, make temporiry file handle.
  *   return: file handle
  *
