@@ -35,6 +35,13 @@
 #define  open(file, flag, mode) PerlLIO_open3(file, flag, mode)
 #endif
 
+/* Loading dynamic cascci library needs this header. */
+#ifdef WIN32
+#include <windows.h>
+#else
+#include <dlfcn.h>
+#endif
+
 #define CUBRID_ER_MSG_LEN 1024
 #define CUBRID_BUFFER_LEN 4096
 
@@ -47,9 +54,17 @@ static struct _error_message {
     {CUBRID_ER_WRITE_FILE, "Cannot write file"},
     {CUBRID_ER_READ_FILE, "Cannot read file"},
     {CUBRID_ER_NOT_LOB_TYPE, "Not a lob type, can only support SQL_BLOB or SQL_CLOB"},
+    {CUBRID_ER_INVALID_PARAM, "Invalid parameter"},
+    {CUBRID_ER_ROW_INDEX_EXCEEDED, "Row index exceeds the allowed range(1 ~ the number of affected rows)"},
+    {CUBRID_ER_EXPORT_NULL_LOB_INVALID, "Exporting NULL LOB is invalid"},
     {0, ""}
 };
 
+static void *libcascci = NULL;
+typedef int (*CCI_GET_LAST_INSERT_ID) (int con, void *buff,
+                                       T_CCI_ERROR * err_buf);
+static CCI_GET_LAST_INSERT_ID cci_get_last_insert_id_fp = NULL;
+
 
 /***************************************************************************
  * Private function prototypes
@@ -112,6 +127,26 @@ dbd_init( dbistate_t *dbistate )
 {
     DBIS = dbistate;
     cci_init();
+
+    /* 
+     * use cci_get_last_id if possible
+     * early distributed libraries don't include it
+     */
+#ifdef WIN32
+    libcascci = LoadLibrary ("cascci.dll");
+
+    if (libcascci != NULL) {
+        cci_get_last_insert_id_fp =
+            (CCI_GET_LAST_INSERT_ID) GetProcAddress (libcascci,
+                                                 "cci_get_last_insert_id");
+    }
+#else
+    libcascci = dlopen ("libcascci.so", RTLD_LAZY);
+
+    if (libcascci != NULL) {
+        cci_get_last_insert_id_fp = dlsym (libcascci, "cci_get_last_insert_id");
+    }
+#endif
 }
 
 /***************************************************************************
@@ -138,6 +173,14 @@ dbd_discon_all( SV *drh, imp_drh_t *imp_drh )
         return FALSE;
     }
 
+    if (libcascci != NULL) {
+#ifdef WIN32
+        FreeLibrary (libcascci);
+#else
+        dlclose (libcascci);
+#endif
+    }
+
     return FALSE;
 }
 
@@ -229,29 +272,24 @@ dbd_db_login6( SV *dbh, imp_dbh_t *imp_dbh,
     int  con, res;
     T_CCI_ERROR error;
 
-#ifdef WINDOWS
-    if ((con = cci_connect_with_url (dbname, uid, pwd)) < 0) {
-        handle_error (dbh, con, NULL);
-        return FALSE;
-    }
-#else
-    if ((con = cci_connect_with_url_ex (dbname, uid, pwd, &error)) < 0) {
+    if ((con = cci_connect_with_url_ex (dbname, uid, pwd, &error)) < 0)
+      {
         handle_error (dbh, con, &error);
         return FALSE;
-    }
-#endif
+      }
 
     imp_dbh->handle = con;
 
-    if ((res = cci_end_tran (con, CCI_TRAN_COMMIT, &error)) < 0) {
+    if ((res = cci_end_tran (con, CCI_TRAN_COMMIT, &error)) < 0)
+      {
         handle_error (dbh, res, &error);
         return FALSE;
-    }
+      }
 
     DBIc_IMPSET_on(imp_dbh);
     DBIc_ACTIVE_on(imp_dbh);
 
-    return TRUE;
+    return TRUE;  
 }
 
 /***************************************************************************
@@ -369,20 +407,22 @@ dbd_db_STORE_attrib( SV *dbh, imp_dbh_t *imp_dbh, SV *keysv, SV *valuesv )
 
     switch (kl) {
     case 10:
-        if (strEQ("AutoCommit", key)) {
-            if (on) {
+        if (strEQ("AutoCommit", key)) 
+        {
+            if (on)
+            {
                 cci_set_autocommit (imp_dbh->handle, CCI_AUTOCOMMIT_TRUE);
             }
-            else {
+            else 
+            {
                 cci_set_autocommit (imp_dbh->handle, CCI_AUTOCOMMIT_FALSE);
             }
 
             DBIc_set (imp_dbh, DBIcf_AutoCommit, on);
+            return TRUE;
         }
-        break;
     }
-
-    return TRUE;
+    return FALSE;
 }
 
 /***************************************************************************
@@ -444,7 +484,15 @@ dbd_db_last_insert_id( SV *dbh, imp_dbh_t *imp_dbh,
     int res;
     T_CCI_ERROR error;
 
-    if ((res = cci_last_insert_id (imp_dbh->handle, &name, &error))) {
+    if (cci_get_last_insert_id_fp != NULL) {
+        /* cci_get_last_id set last_id as con_handle's static buffer */
+        res = cci_get_last_insert_id (imp_dbh->handle, &name, &error);
+    } else {
+        /* cci_last_id set last_id as allocated string */
+        res = cci_last_insert_id (imp_dbh->handle, &name, &error);
+    }
+    
+    if (res < 0) {
         handle_error (dbh, res, &error);
         return Nullsv;
     }
@@ -453,7 +501,13 @@ dbd_db_last_insert_id( SV *dbh, imp_dbh_t *imp_dbh,
         return Nullsv;
     } else {
         sv = newSVpvn (name, strlen(name));
-        free (name);
+
+        /* free last_id which is allocated in cci_last_insert_id */
+        if (cci_get_last_insert_id_fp == NULL) {
+#ifndef WIN32
+            free(name);
+#endif
+        }
     }
 
     return sv_2mortal (sv);
@@ -477,21 +531,19 @@ dbd_db_ping( SV *dbh )
 {
     int res;
     T_CCI_ERROR error;
-    char *query = "SELECT 1+1 from db_root";
+    char *query = "SELECT 1+1 from db_root /*+ shard_id(0) */";
     int req_handle = 0, result = 0, ind = 0;
 
     D_imp_dbh (dbh);
 
     if ((res = cci_prepare (imp_dbh->handle, query, 0, &error)) < 0) {
-        handle_error (dbh, res, &error);
-        return FALSE;
+		goto ER_DB_PING;
     }
 
     req_handle = res;
 
     if ((res = cci_execute (req_handle, 0, 0, &error)) < 0) {
-        handle_error (dbh, res, &error);
-        return FALSE;
+        goto ER_DB_PING;
     }
 
     while (1) {
@@ -500,18 +552,15 @@ dbd_db_ping( SV *dbh )
             break;
         }
         if (res < 0) {
-            handle_error (dbh, res, &error);
-            return FALSE;
+            goto ER_DB_PING;
         }
 
         if ((res = cci_fetch (req_handle, &error)) < 0) {
-            handle_error (dbh, res, &error);
-            return FALSE;
+            goto ER_DB_PING;
         }
 
         if ((res = cci_get_data (req_handle, 1, CCI_A_TYPE_INT, &result, &ind)) < 0) {
-            handle_error (dbh, res, &error);
-            return FALSE;
+            goto ER_DB_PING;
         }
 
         if (result == 2) {
@@ -519,8 +568,9 @@ dbd_db_ping( SV *dbh )
             return TRUE;
         }
     }
-
-    cci_close_req_handle (req_handle);
+	cci_close_req_handle (req_handle);
+ER_DB_PING:
+    handle_error (dbh, res, &error);
     return FALSE;
 }
 
@@ -668,13 +718,11 @@ dbd_st_fetch( SV *sth, imp_sth_t *imp_sth )
     if (res == CCI_ER_NO_MORE_DATA) {
         return Nullav;
     } else if (res < 0) {
-        handle_error (sth, res, &error);
-        return Nullav;
+        goto ERR_ST_FETCH;
     }
 
     if ((res = cci_fetch (imp_sth->handle, &error)) < 0) {
-        handle_error (sth, res, &error);
-        return Nullav;
+        goto ERR_ST_FETCH;
     }
 
     av = DBIS->get_fbav(imp_sth);
@@ -683,17 +731,18 @@ dbd_st_fetch( SV *sth, imp_sth_t *imp_sth )
                                   imp_sth->col_count, 
                                   imp_sth->col_info, 
                                   &error)) < 0) {
-        handle_error (sth, res, &error);
-        return Nullav;
+        goto ERR_ST_FETCH;
     }
 
     res = cci_cursor (imp_sth->handle, 1, CCI_CURSOR_CURRENT, &error);
     if (res < 0 && res != CCI_ER_NO_MORE_DATA) {
-        handle_error (sth, res, &error);
-        return Nullav;
+        goto ERR_ST_FETCH;
     }
 
     return av;
+ERR_ST_FETCH:
+    handle_error (sth, res, &error);
+    return Nullav;	  
 }
 
 /***************************************************************************
@@ -886,13 +935,6 @@ int
 dbd_st_blob_read(SV *sth, imp_sth_t *imp_sth,
         int field, long offset, long len, SV *destrv, long destoffset)
 {
-    sth = sth;
-    imp_sth = imp_sth;
-    field = field;
-    offset = offset;
-    len = len;
-    destrv = destrv;
-    destoffset = destoffset;
     return FALSE;
 }
 
@@ -916,6 +958,100 @@ dbd_st_rows( SV * sth, imp_sth_t * imp_sth )
 {
     return imp_sth->affected_rows;
 }
+/***************************************************************************
+ *
+ * Name:    cubrid_str2bit
+ *
+ * Purpose: convert string to bit
+ *
+ **************************************************************************/
+static char* cubrid_str2bit(char* str)
+{
+    int i=0,len=0,t=0;
+    char* buf=NULL;
+    int shift = 8;
+
+    if(str == NULL)
+        return NULL;
+    len = strlen(str);
+
+    if(0 == len%shift)
+        t =1;
+
+    buf = (char*)malloc(len/shift+1+1);
+    memset(buf,0,len/shift+1+1);
+
+    for(i=0;i<len;i++)
+    {
+        if(str[len-i-1] == '1')
+        {
+            buf[len/shift - i/shift-t] |= (1<<(i%shift)); 
+        }
+        else if(str[len-i-1] == '0')
+        {
+        	//nothing
+        }
+        else
+        {
+            return NULL;
+        }
+    }
+    return buf;
+}
+
+/***************************************************************************
+ *
+ * Name:    _cubrid_dup_buf
+ *
+ * Purpose: copy data to new buf
+ *
+ **************************************************************************/
+
+static char* _cubrid_dup_buf(char* src_buf,int size)
+{
+    int len=0;
+    char* temp_buf=NULL;
+
+    if(src_buf != NULL)
+    {
+        len = strlen(src_buf);
+    }
+    else
+    {
+        len =size;
+    }
+    if(len<=0)
+    {
+        return NULL;
+    }
+    temp_buf = (char*)malloc(len+1);
+    if(NULL==temp_buf)
+    {
+      return NULL;
+    }
+    memset(temp_buf,0,len+1);
+    if(NULL!= src_buf)
+        memcpy(temp_buf,src_buf,len);   
+
+    return temp_buf;
+}
+/***************************************************************************
+ *
+ * Name:    _cubrid_get_data_buf
+ *
+ * Purpose: alloc buf base cubrid data type
+ *
+ **************************************************************************/
+static char* _cubrid_get_data_buf(int type,int num,imp_sth_t *imp_sth)
+{
+    switch(type)
+    {
+        case SQL_BIT:  
+           return  _cubrid_dup_buf(NULL,sizeof(T_CCI_BIT)* (num+1));
+        default:
+           return  _cubrid_dup_buf(NULL,sizeof(void*)* (num+1));
+    } 
+}
 
 /***************************************************************************
  *
@@ -941,21 +1077,106 @@ int
 dbd_bind_ph( SV *sth, imp_sth_t *imp_sth, SV *param, SV *value,
              IV sql_type, SV *attribs, int is_inout, IV maxlen )
 {
-    int res;
-    char *bind_value = NULL;
+    int res = 0,i=0,num=0,buffer_is_null;
+    int* indicator= NULL;   
+    char *bind_value = NULL,*temp=NULL;
     STRLEN bind_value_len;
     T_CCI_ERROR error;
+    AV* aTemp=NULL;
+    SV** element=NULL;
+    T_CCI_SET set;
+    char** potinter=NULL;  
+    T_CCI_U_TYPE u_type = CCI_U_TYPE_STRING;
+    T_CCI_BIT Bit_Val={NULL};   
 
     int index = SvIV(param);
     if (index < 1 || index > DBIc_NUM_PARAMS(imp_sth)) {
         handle_error (sth, CCI_ER_BIND_INDEX, NULL);
         return FALSE;
+    }  
+    
+    buffer_is_null = !(SvOK(newSVsv(value)) && newSVsv(value));
+    if(!buffer_is_null)
+   {
+       if(SvROK(value) && SvTYPE(SvRV(value))==SVt_PVAV)
+       {
+            aTemp = SvRV(value);
+            num = av_len(aTemp);
+
+            potinter = (char**)_cubrid_get_data_buf(sql_type,num+1,imp_sth);
+            indicator  = (int*)_cubrid_dup_buf(NULL,sizeof(int)* (num+1));
+            if(potinter ==NULL || indicator==NULL)
+            {
+                handle_error (sth, CCI_ER_NO_MORE_MEMORY, NULL);
+                return FALSE;                
+            }
+            
+            for(i=0;i<=num;i++)
+            {
+                element = av_fetch(aTemp, i, 0);      
+                temp= SvPV(*element,bind_value_len);
+                if(temp ==NULL)
+                {
+                    handle_error (sth, CCI_ER_NO_MORE_MEMORY, NULL);
+                    return FALSE;                        
+                }
+
+                if(strcmp(temp,"NULL")==0)
+                {
+                    indicator[i]=1;              
+                }
+                else
+                {
+                    indicator[i]=0;
+                    if(SQL_BIT == sql_type)
+                    {
+                        u_type = CCI_U_TYPE_BIT;
+                        temp =  cubrid_str2bit(SvPV(*element,bind_value_len));
+                        if(temp == NULL)
+                        {
+                            handle_error (sth, CCI_ER_TYPE_CONVERSION, NULL);
+                            return FALSE;                             
+                        }
+                        //Bit_Val = ((T_CCI_BIT *) potinter)[i];
+                        ((T_CCI_BIT *) potinter)[i].buf = temp;
+                        ((T_CCI_BIT *) potinter)[i].size = strlen(temp);   
+                    }
+                    else
+                    {
+                        potinter[i]  = temp;
+                    }
+                }    
+            }
+            if(cci_set_make(&set, u_type, num+1, potinter, (int*)indicator)<0)
+            {
+                handle_error (sth, CCI_ER_TYPE_CONVERSION, NULL);
+                return FALSE;    
+            } 
+       }
+       else
+       {
+           bind_value = SvPV (value, bind_value_len);
+       }
+   }
+
+    if (SvOK(value) &&
+            (sql_type == SQL_NUMERIC  ||
+             sql_type == SQL_DECIMAL  ||
+             sql_type == SQL_INTEGER  ||
+             sql_type == SQL_SMALLINT ||
+             sql_type == SQL_FLOAT    ||   
+             sql_type == SQL_REAL     ||   
+             sql_type == SQL_DOUBLE) )
+    {
+        if (! looks_like_number(value)) {
+            handle_error(sth, CUBRID_ER_INVALID_PARAM, NULL);
+            return FALSE;
+        }
     }
 
-    bind_value = SvPV (value, bind_value_len);
-
-    if (sql_type == SQL_BLOB || sql_type == SQL_CLOB) {
-
+    switch(sql_type) {
+    case SQL_BLOB:
+    case SQL_CLOB:
         if ((res = _cubrid_lob_bind (sth, 
                                      index, 
                                      sql_type, 
@@ -964,18 +1185,58 @@ dbd_bind_ph( SV *sth, imp_sth_t *imp_sth, SV *param, SV *value,
             handle_error (sth, res, &error);
             return FALSE;
         }
-
         return TRUE;
+
+    case SQL_NUMERIC:
+        u_type = CCI_U_TYPE_NUMERIC;
+        break;
+
+    case SQL_INTEGER:
+        u_type = CCI_U_TYPE_INT;
+        break;
+
+    case SQL_SMALLINT:
+        u_type = CCI_U_TYPE_SHORT;
+        break;
+
+    case SQL_BIGINT:
+        u_type = CCI_U_TYPE_BIGINT;
+        break;
+
+    case SQL_TINYINT:
+        u_type = CCI_U_TYPE_SHORT;
+        break;
+    case SQL_ARRAY:
+            u_type = CCI_U_TYPE_SET;
+            break;
+
+    default:
+        u_type = CCI_U_TYPE_CHAR;
+        break;
     }
 
+    if (! buffer_is_null) 
+    {
+        if(SvROK(value) && SvTYPE(SvRV(value))==SVt_PVAV)
+        {
+            res = cci_bind_param (imp_sth->handle, index, CCI_A_TYPE_SET, set, CCI_U_TYPE_SET, 0);            
+            cci_set_free (set);        
+        }
+        else
+        {
+            res = cci_bind_param (imp_sth->handle, index, CCI_A_TYPE_STR, bind_value, u_type, 0);
+        }
+    } 
+    else 
+    {
+        res = cci_bind_param (imp_sth->handle, index, CCI_A_TYPE_STR, NULL, u_type, 0);
+    }
 
-    if ((res = cci_bind_param (imp_sth->handle, 
-                    index, CCI_A_TYPE_STR,
-                    bind_value, CCI_U_TYPE_CHAR, 0)) < 0) {
+    if (res < 0) {
         handle_error(sth, res, NULL);
         return FALSE;
     }
-
+    
     return TRUE;
 }
 
@@ -994,7 +1255,6 @@ dbd_bind_ph( SV *sth, imp_sth_t *imp_sth, SV *param, SV *value,
  *
  **************************************************************************/
 
-#ifdef WINDOWS
 SV* dbd_db_quote(SV *dbh, SV *str, SV *type)
 {
     dTHX;
@@ -1012,65 +1272,12 @@ SV* dbd_db_quote(SV *dbh, SV *str, SV *type)
     {
         char *escaped_string = NULL, *unescaped_string = NULL;
         STRLEN len;
-        STRLEN escaped_str_len = 0;
+
         int i = 0;
         char *s = NULL;
-
-        D_imp_dbh(dbh);
-
-        unescaped_string = SvPV (str, len);
-        escaped_string = (char *) malloc (len * 2 + 16);
-        if (!escaped_string)
-        {
-            handle_error (dbh, CCI_ER_NO_MORE_MEMORY, NULL);
-            return Nullsv;
-        }
-
-        memset (escaped_string, 0, len * 2 + 16);
-
-        escaped_string[0] = '\'';
-
-        s = escaped_string + 1;
-
-
-        for (i = 0; i < len; i++) {
-            if (unescaped_string[i] == '\'') {
-                *s++ = '\'';
-                *s++ = '\'';
-                escaped_str_len = escaped_str_len + 2;
-            } else {
-                *s++ = unescaped_string[i];
-                escaped_str_len++;
-            }
-        }
-        *s++ = '\'';
-        *s = '\0';
-        escaped_str_len = escaped_str_len + 2;
-
-        result = newSVpvn (escaped_string, escaped_str_len);
-        free (escaped_string);
-    }
-
-    return result;
-}
-#else
-SV* dbd_db_quote(SV *dbh, SV *str, SV *type)
-{
-    dTHX;
-    SV *result;
-
-    T_CCI_ERROR error;
-    int res;
-
-    if (SvGMAGICAL(str))
-        mg_get(str);
-
-    if (!SvOK(str))
-        return Nullsv;
-    else
-    {
-        char *escaped_string = NULL, *unescaped_string = NULL;
-        STRLEN len;
+        STRLEN escaped_str_len = 0;
+        char db_ver[16] = {'\0'};
+        int major_ver = 0;
 
         D_imp_dbh(dbh);
 
@@ -1089,20 +1296,70 @@ SV* dbd_db_quote(SV *dbh, SV *str, SV *type)
         if ((res = cci_escape_string (imp_dbh->handle, 
                         escaped_string + 1, unescaped_string, len, &error)) < 0)
         {
-            free(escaped_string);
-            handle_error(dbh, res, &error);
-            return Nullsv;
+            if (res != CAS_ER_PARAM_NAME) {
+
+                free(escaped_string);
+                handle_error(dbh, res, &error);
+                return Nullsv;
+
+            } else {
+
+                res = cci_get_db_version (imp_dbh->handle, db_ver, sizeof (db_ver));
+                if (res < 0) {
+                    handle_error (dbh, res, NULL);
+                    return Nullsv;
+                }
+
+                for (i=0; i< sizeof(db_ver); i++) {
+                    if (db_ver[i] == '.') {
+                        db_ver[i] = '\0';
+                        break;
+                    }
+                }
+
+                major_ver = atoi(db_ver);
+
+                if (major_ver < 9) {
+                    /* CUBRID-8.x not support cci_escape_string, so we do it by ourselves. */
+                    s = escaped_string + 1;
+                    for (i = 0; i < len; i++) {
+                        if (unescaped_string[i] == '\'') {
+                            *s++ = '\'';
+                            *s++ = '\'';
+                            escaped_str_len = escaped_str_len + 2;
+                        } else {
+                            *s++ = unescaped_string[i];
+                            escaped_str_len++;
+                        }
+                    }
+
+                    *s++ = '\'';
+                    *s = '\0';
+                    escaped_str_len = escaped_str_len + 2;
+
+                } else {
+
+                    free(escaped_string);
+                    handle_error(dbh, CAS_ER_PARAM_NAME, &error);
+                    return Nullsv;
+
+                }
+
+            }
+
+        } else {
+
+            escaped_string[1 + res] = '\'';
+            escaped_str_len = res + 2;
+
         }
 
-        escaped_string[1 + res] = '\'';
-
-        result = newSVpvn (escaped_string, res + 2);
+        result = newSVpvn (escaped_string, escaped_str_len);
         free (escaped_string);
     }
 
     return result;
 }
-#endif
 
 
 /* Large object functions */
@@ -1130,66 +1387,16 @@ cubrid_st_lob_get( SV *sth, int col )
         return FALSE;
     }
 
-    res = cci_cursor (imp_sth->handle, 0, CCI_CURSOR_CURRENT, &error);
-    if (res == CCI_ER_NO_MORE_DATA) {
-        return TRUE;
-    }
-    else if (res < 0) {
-        handle_error (sth, res, &error);
-        return FALSE;
-    }
-
     u_type = CCI_GET_RESULT_INFO_TYPE (imp_sth->col_info, col);
     if (!(u_type == CCI_U_TYPE_BLOB ||  u_type == CCI_U_TYPE_CLOB)) {
         handle_error (sth, CUBRID_ER_NOT_LOB_TYPE, NULL);
         return FALSE;
     }
 
-    imp_sth->lob = (T_CUBRID_LOB *) malloc (
-            imp_sth->affected_rows * sizeof (T_CUBRID_LOB)
-            );
+    imp_sth->col_selected = col;
 
-    while (1) {
-
-        if ((res = cci_fetch(imp_sth->handle, &error)) < 0) {
-            handle_error (sth, res, &error);
-            return FALSE;
-        }
-
-        if ( u_type == CCI_U_TYPE_BLOB) {
-            imp_sth->lob[i].type = CCI_U_TYPE_BLOB;
-            if ((res = cci_get_data (imp_sth->handle,
-                                     col,
-                                     CCI_A_TYPE_BLOB,
-                                     (void *)&imp_sth->lob[i].lob,
-                                     &ind)) < 0) {
-                handle_error (sth, res, NULL);
-                return FALSE;
-            }
-        }
-        else {
-            imp_sth->lob[i].type = CCI_U_TYPE_BLOB;
-            if ((res = cci_get_data (imp_sth->handle,
-                                     col,
-                                     CCI_A_TYPE_CLOB,
-                                     (void *)&imp_sth->lob[i].lob,
-                                     (&ind))) < 0) {
-                handle_error (sth, res, NULL);
-                return FALSE;
-            }
-        }
-
-        i++;
-
-        res = cci_cursor (imp_sth->handle, 1, CCI_CURSOR_CURRENT, &error);
-        if (res == CCI_ER_NO_MORE_DATA) {
-            break;
-        }
-        else if (res < 0) {
-            handle_error (sth, res, &error);
-            return FALSE;
-        }
-    }
+    imp_sth->lob = (T_CUBRID_LOB *) malloc (imp_sth->affected_rows * sizeof (T_CUBRID_LOB));
+    memset(imp_sth->lob, 0, imp_sth->affected_rows * sizeof (T_CUBRID_LOB));
 
     return TRUE;
 }
@@ -1201,9 +1408,72 @@ cubrid_st_lob_export( SV *sth, int index, char *filename )
     int fd, res, size;
     long long pos = 0, lob_size;
     T_CCI_ERROR error;
+    T_CCI_U_TYPE u_type;
+    int ind = 0;
 
     D_imp_sth (sth);
 
+    if (imp_sth->lob == NULL) {
+        handle_error (sth, CUBRID_ER_CANNOT_FETCH_DATA, NULL);
+        return FALSE;
+    }
+
+    if (index > imp_sth->affected_rows || index < 1) {
+        handle_error (sth, CUBRID_ER_ROW_INDEX_EXCEEDED, NULL);
+        return FALSE;
+    }
+
+    if (imp_sth->col_selected < 1 || imp_sth->col_selected > DBIc_NUM_FIELDS (imp_sth)) {
+        handle_error (sth, CCI_ER_COLUMN_INDEX, NULL);
+        return FALSE;
+    }
+
+    u_type = CCI_GET_RESULT_INFO_TYPE (imp_sth->col_info, imp_sth->col_selected);
+    if (!(u_type == CCI_U_TYPE_BLOB ||  u_type == CCI_U_TYPE_CLOB)) {
+        handle_error (sth, CUBRID_ER_NOT_LOB_TYPE, NULL);
+        return FALSE;
+    }
+
+    res = cci_cursor (imp_sth->handle, index, CCI_CURSOR_FIRST, &error);
+    if (res == CCI_ER_NO_MORE_DATA) {
+        return TRUE;
+    } else if (res < 0) {
+        handle_error (sth, res, &error);
+        return FALSE;
+    }
+
+    if ((res = cci_fetch(imp_sth->handle, &error)) < 0) {
+        handle_error (sth, res, &error);
+        return FALSE;
+    }
+
+    if ( u_type == CCI_U_TYPE_BLOB) {
+        imp_sth->lob[index-1].type = CCI_U_TYPE_BLOB;
+        if ((res = cci_get_data (imp_sth->handle,
+                        imp_sth->col_selected,
+                        CCI_A_TYPE_BLOB,
+                        (void *)&imp_sth->lob[index-1].lob,
+                        &ind)) < 0) {
+            handle_error (sth, res, NULL);
+            return FALSE;
+        }
+    } else {
+        imp_sth->lob[index-1].type = CCI_U_TYPE_BLOB;
+        if ((res = cci_get_data (imp_sth->handle,
+                        imp_sth->col_selected,
+                        CCI_A_TYPE_CLOB,
+                        (void *)&imp_sth->lob[index-1].lob,
+                        (&ind))) < 0) {
+            handle_error (sth, res, NULL);
+            return FALSE;
+        }
+    }
+
+    if ( ind == -1 ) {
+        handle_error (sth, CUBRID_ER_EXPORT_NULL_LOB_INVALID, NULL);
+        return FALSE;
+    }
+
     if ( imp_sth->lob == NULL || imp_sth->lob[index-1].lob == NULL) {
         handle_error (sth, CCI_ER_INVALID_LOB_HANDLE, NULL);
         return FALSE;
@@ -1281,6 +1551,13 @@ cubrid_st_lob_import( SV *sth,
         return FALSE;
     }
 
+    if ((fd = open (filename, O_RDONLY, 0400)) < 0) {
+        res = CCI_ER_FILE;
+        handle_error(sth, res, NULL);
+
+        return FALSE;
+    }
+
     if ((res = _cubrid_lob_new (imp_sth->conn, 
                                 &lob,
                                 u_type, 
@@ -1289,11 +1566,6 @@ cubrid_st_lob_import( SV *sth,
         return FALSE;
     }
 
-    if ((fd = open (filename, O_RDONLY, 0400)) < 0) {
-        res = CCI_ER_FILE;
-        goto ER_LOB_IMPORT;
-    }
-
     while (1) {
         if ((size = read (fd, buf, CUBRID_BUFFER_LEN)) < 0) {
             res = CUBRID_ER_READ_FILE;
